#!/usr/bin/python2

# PyLogger (pylogger.py)
#
# Blogging Tool written in Python and distributed under the
# terms of the LGPL.
#
# Written by A.Sajjad Zaidi (http://www.sajjadzaidi.com/)
# Copyright April 2003
#

import os,sys
import string
from string import atoi
import re
import shutil
import processWeb


# Get configuration
confFile = '/etc/pylogger.conf'
try:
	c = open(confFile)
	readFile = c.readlines()
	c.close()
except:
	print 'Error opening conf file %s' % confFile
	sys.exit(1)
confContents = string.join(readFile,'')

locRootPat = re.compile(r'^localroot\s*=\s*\"?(\S*)\"?')
testRootPat = re.compile(r'^testroot\s*=\s*\"?(\S*)\"?')



def parseCGIForm():
	"""Parse CGI Form and display main page if no arguments given"""

	import cgi

	form = cgi.FieldStorage()

	if form.has_key('cancel'):
		printMainPage()
		sys.exit(0)
	elif form.has_key('preview'):
		blSubject = form['subject'].value
		blBody = form['blogbody'].value
		printPreviewPage(blSubject, blBody)
	elif form.has_key('confirm'):
		blSubject = form['subject'].value
		blBody = form['blogbody'].value
		addNewEntry(blSubject, blBody)
	else:
		printMainPage()
		sys.exit(0)
	return()





def parseShellCmd():
	"""If command line arguments are given, parse them and return the
	results. This seems to work, but all the required options must
	be given"""

	from getopt import getopt

	allCmdArgs = sys.argv
	cmdUsage = 'Usage: %s [OPTIONS]\n\
Command line blog entries.\n\
   -h, --help			display this help and exit\n\
   -s, --subject=SUBJECT	Use this as the subject of the blog entry\n\
   -b, --blogdata=DATA		The content of the blog entry\n\
   -f, --file=FILENAME		Read this file and use contents as blog body' % sys.argv[0]
	allCmdArgs.pop(0)
	shortCmdOpts = "hs:b:f:"
	longCmdOpts = ['help','subject=','blogdata=','file=']
	try:
		opts, args = getopt(allCmdArgs,shortCmdOpts,longCmdOpts)
		for opt, val in opts:
			if opt == "-h" or opt == "--help":
				sys.exit(1)
			if opt == "-s" or opt == "--subject":
				blSubject = val
			if opt == "-b" or opt == "--blogdata":
				blBody = val
			if opt == "-f" or opt == "--file":
				blFile = val
				try:
					bf = open(blFile)
					blBody = string.join(bf.readlines(),'')
					bf.close()
				except:
					print 'Error opening blog file %s' % blFile
					sys.exit(1)
	except:
		print cmdUsage
		sys.exit(1)

	# Try to get the blog's subject from <subject> tags in the blog
	# body.
	sPat = re.compile(r'<subject>([^<]*)</subject>')
        subjectMatch = sPat.search(blBody)
        if subjectMatch:
                blSubject = subjectMatch.group(1)
                blBody = string.replace(blBody, '<subject>%s</subject>' % blSubject, '')

	addNewEntry(blSubject, blBody)
	return()





def promptForData():
	"""Get blog subject and body interactively through the
	shell"""

	blSubject = raw_input('Enter Subject: ')
	blBody = raw_input('Enter Body of Entry:\n')

	print
	print
	print 'You entered the following:'
	print
	print 'Subject: %s' % blSubject
	print 'Body: %s' % blBody

	print
	confirmResponse = raw_input('Enter y to continue: ')

	if confirmResponse == 'y':
		addNewEntry(blSubject, blBody)
	else:
		sys.exit(0)
	return()





def printMainPage():
	"""Print the main blog input CGI page"""

        print 'Content-type: text/html\n\n'
        print '''
<html>
        <head>
                <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
                <title>Blog Updater</title>
                <link href="/mainstyles.css" type="text/css" rel="stylesheet">
        </head>
        <body>
                <form action="/pylogger.py" method="post">
                        <table class="stats">
                                <tr>
                                        <th class="title"></th>
                                </tr>
                                <tr>
                                        <td class="options">
                                                <input name="subject" size="72">
                                        </td>
                                </tr>
                                <tr>
                                        <td class="options">
                                                <textarea cols="72" rows="20" name="blogbody"></textarea>
                                        </td>
                                </tr>
                                <tr>
                                        <td class="options">
                                                <input value="Preview" name="preview" type="submit"><input value="Clear" name="clear" type="submit">
                                        </td>
                                </tr>
                        </table>
                </form>
                <p></p>
        </body>
</html>'''
        return()




def printPreviewPage(blSubject, blBody):
	"""Generate preview of blog posting"""

	#escSubject = string.replace(blSubject,'"','&dq;')
	#escSubject = string.replace(escSubject,'>','&gt;')
	#escBody = blBody
	#escBody = string.replace(blBody,'"','\&quot;')
	escBody = string.replace(blBody,'<','&lt;')
	escBody = string.replace(escBody,'>','&gt;')
        print 'Content-type: text/html\n\n'
        print '''
<html>
        <head>
                <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
                <title>Blog Updater Preview</title>
                <link href="/mainstyles.css" type="text/css" rel="stylesheet">
        </head>
        <body>
                <form action="/pylogger.py" method="post">
                        <table class="stats">
                                <tr>
                                        <th class="title"></th>
                                </tr>
                                <tr>
                                        <td class="options">
						<p></p><h5>Tuesday 1st April 2003</h5>
						<p><b>%s:</b><br></br>
						%s
						</p>
						<p class="blogend">12:51pm <a href="">plink</a></p>
						<p></p>
						<hr></hr>
                                        </td>
                                </tr>
                                <tr>
                                        <td class="options">
                                                <input name="subject" size="72" value="%s">
                                        </td>
                                </tr>
                                <tr>
                                        <td class="options">
                                                <textarea cols="72" rows="20" name="blogbody">%s</textarea>
                                        </td>
                                </tr>
                                <tr>
                                        <td class="options">
                                                <input value="Review" name="preview" type="submit"><input value="Confirm" name="confirm" type="submit"><input value="Cancel" name="cancel" type="submit">
                                        </td>
                                </tr>
                        </table>
                </form>
                <p></p>
        </body>
</html>''' % (blSubject, blBody, blSubject, escBody)
        return()





def getTimes():
	"""Generate the desired date and time formats for the blog entry"""

	import time

	dayOfWeek = time.strftime("%A", time.localtime(time.time()))

	currentHour = time.strftime("%H", time.localtime(time.time()))
	if atoi(currentHour) < 12:
		timeSuffix = 'am'
	else:
		timeSuffix = 'pm'

	currentMinute = time.strftime("%M", time.localtime(time.time()))

	dayOfMonth = time.strftime("%d", time.localtime(time.time()))
	if atoi(dayOfMonth) < 10:
		dayOfMonth = string.replace(dayOfMonth,'0','')
	if dayOfMonth == '1':
		dayOfMonth = '1st';
	elif dayOfMonth == '2':
		dayOfMonth = '2nd';
	elif dayOfMonth == '3':
		dayOfMonth = '3rd';
	elif dayOfMonth == '21':
		dayOfMonth = '21st';
	elif dayOfMonth == '22':
		dayOfMonth = '22nd';
	elif dayOfMonth == '23':
		dayOfMonth = '23rd';
	elif dayOfMonth == '31':
		dayOfMonth = '31st';
	else:
		dayOfMonth = '%sth' % dayOfMonth
	monthOfYear = time.strftime("%B", time.localtime(time.time()))
	decimalYear = time.strftime("%Y", time.localtime(time.time()))
	shortMonth = time.strftime("%b", time.localtime(time.time()))

	blogDate = '%s %s %s %s' % (dayOfWeek, dayOfMonth, monthOfYear, decimalYear)
	blogTime = '%s:%s%s' % (currentHour, currentMinute, timeSuffix)

	return(blogDate, blogTime, shortMonth)






def addNewEntry(blogSubject, blogBody):
	"""Get input from somewhere and insert it in the main page, together with
	the time stamp"""

	allTimes = getTimes()

	blogDate = allTimes[0]
	blogTime = allTimes[1]
	blogMonth = string.lower(allTimes[2])

	try:
		fp = open('index.asz')
		fullFile = string.join(fp.readlines(),'')
		fp.close()
	except:
		print 'Error opening index.asz'

	datePat = re.compile(r'<!-- NEW ENTRIES START HERE -->\s*<p></p><h5>(.*?)</h5>\s*<div>\s*<a name="(\d+)">')
	allMatches = datePat.search(fullFile)
	lastBlogDate = allMatches.group(1)
	lastBookMark = allMatches.group(2)
	if atoi(lastBookMark) < 100000:
		newBookMark = atoi(lastBookMark) + 100001
	else:
		newBookMark = atoi(lastBookMark) + 1

	blogSection = '''<!-- NEW ENTRIES START HERE -->
								<p></p><h5>%s</h5>

								<div>
								<a name="%s"></a>
								<p><b>%s:</b><br></br>
%s
								<p class="blogend">Posted at %s <a href="http://www.gangblog.com/cgi-bin/comments.py?blogid=%s">Comments</a> | <a href="/2003/%s/index.html#%s">plink</a></p>
								</div>

''' % (blogDate, newBookMark, blogSubject, blogBody, blogTime, newBookMark, blogMonth, newBookMark)

	if blogDate == lastBlogDate:
		updatedText = string.replace(fullFile,'\t\t\t\t\t\t\t\t<!-- NEW ENTRIES START HERE -->\n', '')
		updatedText = string.replace(updatedText,'<p></p><h5>%s</h5>' % blogDate, blogSection)
	else:
		updatedText = string.replace(fullFile,'<!-- NEW ENTRIES START HERE -->', blogSection)

	w = open('index.asz', 'w')
	w.write(updatedText)
	w.close()

	updateIndexFile(newBookMark,blogDate,blogTime,blogSubject)

	return()



def updateIndexFile(indexNumber,fullDate,blogTime,blogSubject):
	"""Append an entry for this blog into the index file. This
	will work great with a database"""

	indexFile = '/home/sajjad/blogindex'

	fp = open(indexFile)
	oldDataList = fp.readlines()
	fp.close()
	oldData = string.join(oldDataList,'')

	i = open(indexFile,'w')
	i.write(oldData)
	i.write('%s,"%s","%s","%s"\n' %(indexNumber,fullDate,blogTime,blogSubject))
	i.close()
	return()




def generateRSS():
	"""Generate XML from the last 20 entries"""

	global LOCALROOT

	feedFileName = "myfeed.rss"
	f = open("%s" % feedFileName)
	readFeed = f.readlines()
	fullFeed = string.join(readFeed, '')
	f.close()
	destFeedFile = "%s/%s" % (LOCALROOT, feedFileName)
	n = open('%s' % destFeedFile, 'w')

	xmlHeader = '''<?xml version="1.0"?>

<!DOCTYPE rss PUBLIC "-//Netscape Communications//DTD RSS 0.91//EN"
              "http://my.netscape.com/publish/formats/rss-0.91.dtd">

<rss version="0.91">

<channel>

<title>A.Sajjad Zaidi's Blog</title>
<link>http://www.sajjadzaidi.com/</link>
<description>The latest entries</description>
<language>en-us</language>

<image>
        <title>A.Sajjad Zaidi's Blog</title>
        <link>http://www.sajjadzaidi.com/</link>
        <description>The latest entries</description>
        <url>http://www.sajjadzaidi.com/images/tux-small.png</url>
        <width>99</width>
        <height>100</height>
</image>
'''

	xmlFooter = '''<item>
   <title>Main page</title>
   <link>http://www.sajjadzaidi.com/</link>
</item>
</channel></rss>
'''

	rssPat = re.compile(r'<p><b>([^<]+):</b><br></br>\s*(\S.*?)<p class="blogend">Posted at \d\d:\d\d\w\w <a href="[^"]*">Comments[^<]*</a>[^<]*<a href="([^"]*)"')

	fp = open('index.asz')
	indexFile = string.join(fp.readlines(), '')
	fp.close()

	indexFile = string.replace(indexFile,'\t','')
	indexFile = string.replace(indexFile,'\n',' ')

	rssMatches = rssPat.findall(indexFile)

	n.write(xmlHeader)
	n.write('\n')

	count = 20

	for m in rssMatches:
		if count > 0:
			n.write('<item>\n')
			n.write('\t<title>%s</title>\n' % m[0])
			n.write('\t<link>http://www.sajjadzaidi.com%s</link>\n' % m[2])
			n.write('\t<description><![CDATA[<p>%s]]></description>\n' % m[1])
			n.write('</item>\n')
			n.write('\n')
			count = count - 1
		else:
			break

	n.write(xmlFooter)

	n.close()
	return()






def runRsync():
	"""Full update through rsync and ssh"""

	fullWebRoot = '/home/sajjad/work/ahlebait.jp'

	os.chdir('%s' % fullWebRoot)
	updateWeb = os.popen('rsync -Cave ssh --delete . sajjad@wanote.pair.com:public_html')

	updateOutput = string.join(updateWeb.readlines(),'')
	updateWeb.close()

	print updateOutput

	return()



def pingWebLogs():
	"""Ping WebLogs after we update"""

	runWget = os.popen('wget -q -O - -U "Pylogger" "http://newhome.weblogs.com/pingSiteForm?name=Sajjad%27s+Blog&url=http%3A%2F%2Fwww.sajjadzaidi.com%2F"')

	pingOutput = string.join(runWget.readlines(),'')
	runWget.close()
	print pingOutput

	return()





###########################################
##                 Start                 ##
###########################################


#localBlogRoot = '/home/sajjad/work/ahlebait.jp/sajjadzaidi.com'
#testBlogRoot = '/home/sajjad/work/sajjadzaidi.com'


# Store the template in string to be used later
os.chdir('%s' % TESTROOT)

# Check how we were accessesd
if os.environ.has_key("GATEWAY_INTERFACE"):
	parseCGIForm()
elif len(sys.argv) > 1:
	# Not tested so be careful
	parseShellCmd()
else:
	promptForData()


# Generate the final templates and copy over the CSS file
processWeb.tmplGen()
#createFinalPages()


# Generate RSS feed from index.asz
generateRSS()


# Final update to website
processWeb.runRsync()

pingWebLogs()

sys.exit(0)

